/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 */

/*=========================================================================
 * Spotless Virtual Machine
 *=========================================================================
 * SYSTEM:    KVM
 * FILE:      machine_md.h (for UNIX/Solaris)
 * OVERVIEW:  This file is included in every compilation.  It contains
 *            definitions that are specific to the Solaris port of KVM
 * AUTHOR:    Frank Yellin
 * NOTE:      This file overrides many of the default compilation 
 *            flags and macros defined in VmCommon/h/main.h.
 *=======================================================================*/

/*=========================================================================
 * Platform definition
 *=======================================================================*/

#define SOLARIS 1
#define UNIX    1

/*=========================================================================
 * Include files
 *=======================================================================*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <time.h>
#include <math.h>

/*=========================================================================
 * Platform-specific datatype definitions
 *=======================================================================*/

typedef long long           long64;    /*  64-bit signed integer type */
typedef unsigned long long  ulong64;   /*  64-bit unsigned integer type */

#ifndef Calendar_md
unsigned long * Calendar_md(void);
#endif

/*=========================================================================
 * Compilation flags and macros that override values defined in main.h
 *=======================================================================*/

/* This is a big-endian target platform */
#define BIG_ENDIAN 1

/* Solaris requires 8-byte alignment of longs and doubles */
#define NEED_LONG_ALIGNMENT 1
#define NEED_DOUBLE_ALIGNMENT 1

/* Make the VM run a little faster (can afford the extra space) */
#define ENABLEFASTBYTECODES 1

/* Solaris port utilizes the Generic Connection framework */
#define GENERICNETWORK 1
#define GENERICSTORAGE 1

/* Override the sleep function defined in main.h */
#define SLEEP_UNTIL(wakeupTime)                                                         \
    {  long delta = wakeupTime - CurrentTime_md();                                      \
           struct timeval timeout;                                                      \
           timeout.tv_sec = delta / 1000;                                               \
           timeout.tv_usec = (delta % 1000) * 1000;                                     \
           select(0, NULL, NULL, NULL, &timeout);                                       \
        }

/*=========================================================================
 * Platform-specific macros and function prototypes
 *=======================================================================*/

#define InitializeVM()
#define FinalizeVM()

#define freeHeap(heap) free(heap)
#define RandomNumber_md() rand()

void InitializeWindowSystem();
void FinalizeWindowSystem(void);

